`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    17:55:22 05/04/2016 
// Design Name: 
// Module Name:    Controller 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module Controller(
	//Clock Input Pin
	 input 				Clk_100M,
	 
	//Pins to solver
    output 	[127:0] 	MD5HashOutput,
    output 				ReadyFlag,
    input 	[127:0] 	Attempts,
    input 	[79:0] 	Result,
    input 				SolvedFlag,
	 
	//Transmitter Pins
    output 	[7:0] 	TXData,
    output 				SendFlag,
    input 				UART_Sent_Confirmation_Flag,
	 
	//Receiver Pins
    input 				UART_Received_Byte_Ready_Flag,
    output 				ReceivedFlag,
    input 	[7:0] 	UART_Received_Byte_Data
    );


//Count
reg [127:0] NumberOfCombinations;
reg [5:0] CountBytePos = 0;


//UART Variables

reg [7:0]	UARTByteOut = 8'b00000000;//Data to be sent to the UARTTx block
reg [7:0]	UARTByteIn = 8'b00000000;

//UART receving data
wire			UART_Received_Byte_Acknowledge;	
assign 		UART_Received_Byte_Acknowledge = UART_Received_Byte_Ready_Flag;

//UART sending data
reg 			UART_Byte_In_Transit = 1'b0;


//Hash Register
reg [127:0]	MD5Hash = 0; 
reg [7:0]	command = 0;
reg [3:0]	HashByte = 0;
reg 			HashReady = 0;
reg [127:0]	HashBuffer = 0; 

//Register for data back 
reg [79:0]	Password;
reg 			solved;
reg 			solvedSent;
reg [0:5]	countUnitsSent  = 0;

//Test reg
reg [7:0] word;


//Calculations get performed

always @(posedge Clk_100M) begin
	
	//When data is received from the controller
	if(UART_Received_Byte_Ready_Flag == 1)begin
		 //UARTByteOut <= UART_Received_Byte_Data;
		 case(command)
			
			//Command to indicate that the next 16 bytes received are the hash
			8'b00000001: begin
				if(HashByte == 4'b1111)begin
					command <= 8'b00000000;
					HashByte<= 4'b0000;
					HashReady<= 1'b1;
					solvedSent <= 1'b0;
				end else begin
					HashByte <= HashByte + 1'b1;
					HashReady<= 1'b0;
				end
				MD5Hash <= {MD5Hash[119:0],UART_Received_Byte_Data};
			end
			
			8'b00000010: begin
				//Left empty just to ensure the data sent does not change the command
			end
			
			8'b00001000: begin
				//Left empty just to ensure the data sent does not change the command
			end
			
			8'b00001010: begin
				//Left empty just to ensure the data sent does not change the command
			end
			
			8'b00001001: begin
				//Left empty just to ensure the data sent does not change the command
			end
			//If there is no command being processed assume the input data is a command
			default: command <= UART_Received_Byte_Data;
		 endcase
		
	end
	
	if(UART_Byte_In_Transit == 1'b0 && UART_Sent_Confirmation_Flag == 1'b0)begin
		case(command)
			//Command indicating it needs to transfer the count to the controller 
			8'b00000010: begin
				UART_Byte_In_Transit <= 1'b1;
				if(CountBytePos == 5'b00000) begin
					CountBytePos <= CountBytePos + 1;
					UARTByteOut <= 8'b00000011;
				end else begin
					if(CountBytePos == 5'b10000) begin
						CountBytePos <= 0;
						command <= 8'b00000000;
					end else begin
						CountBytePos <= CountBytePos + 1;
					end
					UARTByteOut <= NumberOfCombinations[127:120];
					NumberOfCombinations <= NumberOfCombinations << 8;
				end
			end
			
			//When the CPU requests to find the transfer data
			8'b00001000: begin
				UART_Byte_In_Transit <= 1'b1;
				if(countUnitsSent == 5'b00000)begin
					//Determines if the data has been sent
					if(SolvedFlag == 1) begin
						countUnitsSent <= 5'b00001;
						UARTByteOut <= 8'b00000110;
					end else begin
						UARTByteOut <= 8'b00000111;
						command <= 8'b00000000;
						countUnitsSent <= 5'b00000;
					end
				end else begin
					if(countUnitsSent == 5'b01010) begin
						command <= 8'b00000000;
						countUnitsSent <= 5'b00000;
						solvedSent <= 1'b1;
					end else begin
						countUnitsSent <= countUnitsSent + 5'b00001;
					end
					UARTByteOut <= Password[79:72];
					Password <= Password << 8;
				end
			end
			
			//When the hash needs to be sent back to the controller
			8'b00001010: begin
				UART_Byte_In_Transit <= 1'b1;
				if(CountBytePos == 5'b00000) begin
					CountBytePos <= CountBytePos + 1;
					UARTByteOut <= 8'b00001010;
				end else begin
					if(CountBytePos == 5'b10000) begin
						CountBytePos <= 0;
						command <= 8'b00000000;
					end else begin
						CountBytePos <= CountBytePos + 1;
					end
					UARTByteOut <= HashBuffer[127:120];
					HashBuffer <= HashBuffer << 8;
				end
			end
			
			//Occurs when the program is solved, sends a code to the controlling computer
			/*8'b00001001: begin
				UART_Byte_In_Transit <= 1'b1;
				UARTByteOut <= 8'b00001001;
				solvedSent <= 1'b1;
				command <= 8'b00000000;
			end*/
			
		endcase
	end else begin
		if(UART_Sent_Confirmation_Flag == 1'b1) begin
			UART_Byte_In_Transit <= 1'b0;
		end
	end
	
	//Will lock this value if the command to transfer the number of combinations has been sent
	if(command != 8'b00000010) begin
		NumberOfCombinations <= Attempts;
	end
	
	//Will lock this value if the command to transfer the password is sent.
	if(command != 8'b00001000) begin
		Password<=Result;
	end
	
	//If the calculation module indicates that the password has been found and no other transactions take place, then send the data
	//if(HashReady == 1'b1 && SolvedFlag == 1'b1 && command == 8'b00000000 && solvedSent == 1'b0 && UART_Received_Byte_Ready_Flag!= 1) begin
	//	command <= 8'b00001001;
	//end
	
	if(command != 8'b00001010) begin
		HashBuffer <= MD5Hash;
	end
	
end

assign 	MD5HashOutput	= MD5Hash;
assign	ReadyFlag		= HashReady;	
assign	TXData			= UARTByteOut;
assign	SendFlag			= UART_Byte_In_Transit;
assign	ReceivedFlag	= UART_Received_Byte_Acknowledge;


endmodule
