`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    17:12:02 04/20/2016 
// Design Name: 
// Module Name:    UARTTx 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module UARTTx(
    input [7:0] 	sendData,//This is the data from the connecting entity that will be sent out on the UART line
	 input 			SendFlag,//THis is an input from the connecting entity indicating that the data is ready to be sent out, note: this line will need to be pulled low again before the next bit of data will be sent, ie it will not ctsly send data if it is left high
	 input 			clk,//This is the clock
	 
	 output			UARTOut,//This needs to be connected to the UARTTx line
	 output			sentFlag//This flag indicates to the connecting entity that the data has been sent
	 );

reg [10:0]	Count;
reg [7:0]	UARTByteOut;//Data to be sent next
reg [2:0]	pos = 0;	//bit number to be sent next in UARTByteOut
reg			UARTOutReg = 1;//Signal That is being sent out
reg 			Sending = 0;//Specifies if the signal is sending
reg			Toggled = 0;//Indicates the send command has been sent but has not been Flaged back
reg			Stop = 0;//If 1 the stop bit will be sent next
reg			sentFlagReg = 0;//Register that will be high when the byte has been sent for a single clock cycle

always @(posedge clk) begin
	//Divides the clock speed
	if(Count == 1000) begin
		
		//Determines that it must start sending data - sends the start signal
		if(Sending == 0 && SendFlag == 1 && Toggled == 0) begin
				UARTOutReg <=0;
				Sending <= 1;
				Toggled <= 1;
				UARTByteOut <= sendData;
		end
		
		//Starts sending the byte of data, this block will execute eight times in a row
		if(Sending == 1) begin
			case (pos)
				3'b000: UARTOutReg <= UARTByteOut[0];
				3'b001: UARTOutReg <= UARTByteOut[1];
				3'b010: UARTOutReg <= UARTByteOut[2];
				3'b011: UARTOutReg <= UARTByteOut[3];
				3'b100: UARTOutReg <= UARTByteOut[4];
				3'b101: UARTOutReg <= UARTByteOut[5];
				3'b110: UARTOutReg <= UARTByteOut[6];
				3'b111: begin  //The last bit is sent and stop signal flag is set to high
								UARTOutReg <= UARTByteOut[7];
								Stop 	<= 1'b1;
								Sending <= 1'b0;
						end
			endcase
			pos<=pos+1;
		end
		
		//Sends the stop bit
		if(Stop == 1)begin
			Stop	<=		0;
			UARTOutReg	<= 	1'b1;
			sentFlagReg <=		1'b1;
		end
		
		//Once the data has finished being sent and the send command is set to zero, toggled is set back to zero to be ready for the next send 
		if(SendFlag == 0 && Toggled == 1 && Stop == 0) begin
			Toggled <= 0;
			sentFlagReg <=0;
		end
		
		Count <= 0;
		
	end else begin
		Count <= Count +1'b1;
	end
end

assign UARTOut =	UARTOutReg;
assign sentFlag = sentFlagReg;

endmodule
